import { glob } from 'astro/loaders'
import type { CollectionEntry } from 'astro:content'
import { defineCollection, z } from 'astro:content'

// TYPES
const articleSchema = z.object({
	title: z.string(),
	subtitle: z.string(),
	lang: z.enum(['fr', 'en']),
	tags: z.array(z.string()), // An array of strings
	type: z.string(),
	slug: z.string(),
	// Parse pubDate as a browser-standard `Date` object
	createdAt: z.string().transform((str) => new Date(str)),
	updatedAt: z
		.string()
		.transform((str) => new Date(str))
		.optional(),
	code: z.boolean().optional() || false,
	draft: z.boolean().optional() || false
})

const fragmentSchema = z.object({
	title: z.string(),
	subtitle: z.string(),
	lang: z.enum(['fr', 'en']),
	tags: z.array(z.string()), // An array of strings
	type: z.string(),
	slug: z.string(),
	// Parse pubDate as a browser-standard `Date` object
	createdAt: z.string().transform((str) => new Date(str)),
	updatedAt: z
		.string()
		.transform((str) => new Date(str))
		.optional(),
	code: z.boolean().optional() || false,
	draft: z.boolean().optional() || false
})

const referenceSchema = z.object({
	title: z.string(),
	subtitle: z.string(),
	url: z.string(),
	lang: z.enum(['fr', 'en']),
	slug: z.string(),
	tags: z.array(z.string()), // An array of strings
	// Parse pubDate as a browser-standard `Date` object
	createdAt: z.string().transform((str) => new Date(str)),
	updatedAt: z
		.string()
		.transform((str) => new Date(str))
		.optional(),
	code: z.boolean().optional() || false,
	draft: z.boolean().optional() || false
})

const HPsectionSchema = z.object({
	type: z.string(),
	lang: z.enum(['fr', 'en']),
	uid: z.string().optional(),
	image: z.string().optional(),
	order: z.number(),
	quickTitle: z.string().optional(),
	quickImage: z.string().optional(),
	reference: z.string().optional()
})

const VeilleSchema = z.object({
	lang: z.enum(['fr', 'en']),
	title: z.string(),
	updatedAt: z
		.string()
		.transform((str) => new Date(str))
		.optional()
})

// COLLECTIONS
const articles = defineCollection({
	loader: glob({
		pattern: '**/[^_]*.{md,mdx}',
		base: './src/content/articles'
	}),
	schema: articleSchema
})

const fragments = defineCollection({
	loader: glob({
		pattern: '**/[^_]*.{md,mdx}',
		base: './src/content/fragments'
	}),
	schema: fragmentSchema
})

const references = defineCollection({
	loader: glob({
		pattern: '**/[^_]*.{md,mdx}',
		base: './src/content/references'
	}),
	schema: referenceSchema
})

// DATA
const HPsections = defineCollection({
	loader: glob({ pattern: '**/[^_]*.{md,mdx}', base: './src/data/HP' }),
	schema: HPsectionSchema
})

const veille = defineCollection({
	loader: glob({ pattern: '**/[^_]*.{md,mdx}', base: './src/data/veille' }),
	schema: VeilleSchema
})

export const collections = {
	// Don't forget 'quotes' for collection names containing dashes
	articles,
	fragments,
	references,
	HPsections,
	veille
}

// TYPES from the schemas
export type ArticleFrontmatter = z.infer<typeof articleSchema>
export type FragmentFrontmatter = z.infer<typeof fragmentSchema>
export type ReferenceFrontmatter = z.infer<typeof referenceSchema>
export type HPsectionsFrontmatter = z.infer<typeof HPsectionSchema>
export type VeilleFrontmatter = z.infer<typeof VeilleSchema>

// full entry type including data, body, slug etc.
export type ArticleEntry = CollectionEntry<'articles'>
export type FragmentEntry = CollectionEntry<'fragments'>
export type ReferenceEntry = CollectionEntry<'references'>
export type HPSectionEntry = CollectionEntry<'HPsections'>
